@@ -30,7 +30,7 @@ class ContainerEntry(NamedTuple):
     cleanup: Optional[InstanceCleanup]
     cache: bool
 
-    async def run_finalize(self, instance: Any) -> None:
+    async def run_finalize(self, instance: Any):
         """Run the finalize function on an instance of the service."""
         if not self.finalize:
             return
@@ -39,7 +39,7 @@ class ContainerEntry(NamedTuple):
         if inspect.isawaitable(result):
             await cast(Awaitable[None], result)
 
-    async def run_cleanup(self, instance: Any) -> None:
+    async def run_cleanup(self, instance: Any):
         """Run the cleanup function on an instance of the service."""
         if not self.cleanup:
             return
@@ -75,12 +75,14 @@ class Container:
     async def reset(self, key: Key) -> "Container":
         """Finalize, cleanup, and remove an service from the container instance."""
         key = _get_key(key)
+        await self.finalize(key)
         await self.cleanup(key)
         del self.services[key]
         return self
 
     async def reset_all(self) -> "Container":
         """Finalize, cleanup, and remove all services from the container instance."""
+        await self.finalize_all()
         await self.cleanup_all()
         self.services = {}
         return self
@@ -91,34 +93,36 @@ class Container:
         """
         return await self.reset_all()
 
-    async def finalize(self, key: str) -> None:
-        """Finalize a single service."""
+    async def finalize(self, key: str):
+        """Finalize a single service if it has been booted and it has a finalize function
+        registered.
+        """
         if key not in self.services:
             return
 
         instance = self.services[key]
         await self.container_definition.get(key).run_finalize(instance)
 
-    async def finalize_all(self) -> None:
+    async def finalize_all(self):
         """Iterate through all services that have been created and call the associated finalize
-        methods.
+        method for that service, if there is one.
         """
         for key in self.services:
             await self.finalize(key)
 
-    async def cleanup(self, key: str) -> None:
-        """Finalize and cleanup a single service"""
+    async def cleanup(self, key: str):
+        """Cleanup a single service if it has been booted and it has a cleanup function
+        registered.
+        """
         if key not in self.services:
             return
 
-        entry = self.container_definition.get(key)
         instance = self.services[key]
-        await entry.run_finalize(instance)
-        await entry.run_cleanup(instance)
+        await self.container_definition.get(key).run_cleanup(instance)
 
-    async def cleanup_all(self) -> None:
-        """Iterate through all services that have been created, and call the associated finalize
-        and cleanup methods.
+    async def cleanup_all(self):
+        """Iterate through all services that have been created, and call the associated cleanup
+        method for that service, if there is one.
         """
         for key in self.services:
             await self.cleanup(key)
@@ -252,12 +256,12 @@ class ContainerDefinition:
         self.cleanups.append(cleanup)
         return self
 
-    async def cleanup(self) -> None:
+    async def cleanup(self):
         """Run all registered cleanup methods, in the order they were registered."""
         for cleanup in self.cleanups:
             result = cleanup()
             if inspect.isawaitable(result):
-                await cast(Awaitable[None], result)
+                await result
 
 
 def _key_list_factory(dependencies: Sequence[str], factory: Callable) -> Factory:
